/*
 * main3d.c
 *
 *  Created on: 4 de mar de 2017
 *      Author: Alexandre
 */

#include "main3d.h"

LRESULT CALLBACK MyWndProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam);
pthread_t loop, display;

BOOL fDraw = FALSE;
Vector3d p0;
int xi, xf, yi, yf;
extern int background;

boolean types[9];
int n = 4;
boolean quadrant = false;

void DeleteTree()
{
	free(pixels);
	free(colors);
	free(lxi);
	free(lyi);
	free(lzi);
	free(lxf);
	free(lyf);
	free(lzf);
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
	WNDCLASS wc;
	HWND hwnd;
	MSG msg;
	//
	ZeroMemory(&wc, sizeof(WNDCLASS));
	//
	wc.hInstance     = hInstance;
	wc.lpfnWndProc   = MyWndProc;
	wc.lpszClassName = "MYWNDCLASSNAME";
	wc.hbrBackground = NULL;
	wc.hCursor       = LoadCursor(0, IDC_ARROW);
	//
 	char *title;
 	asprintf(&title, "Tree %dx%dx%d", n, n, n);
 	//
	RegisterClass(&wc);
	hwnd = CreateWindow("MYWNDCLASSNAME", title,
		WS_SYSMENU | WS_MINIMIZEBOX | WS_VISIBLE,
		10, 10, WIDTH + 4, HEIGHT + 4, NULL, NULL, hInstance, NULL);
 	free(title);
	//
 	generateTree();
 	pthread_create(&display, NULL, &DisplayLoop, NULL);
 	sleep(2);
 	//
	while(GetMessage(&msg, hwnd, 0,0))
		DispatchMessage(&msg);
	return 0;
}

LRESULT CALLBACK MyWndProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
	switch(msg)
	{
		case WM_CREATE:
    		hdc = GetDC(hwnd);
    		pixels = (DWORD*) malloc(WIDTH * HEIGHT * sizeof(DWORD));
    		initPlot(pixels);
			ZeroMemory(&bmInfo, sizeof(BITMAPINFO));
			bmInfo.bmiHeader.biBitCount    = 32;
			bmInfo.bmiHeader.biHeight      = HEIGHT;
			bmInfo.bmiHeader.biPlanes      = 1;
			bmInfo.bmiHeader.biSize        = sizeof(BITMAPINFOHEADER);
			bmInfo.bmiHeader.biWidth       = WIDTH;
			bmInfo.bmiHeader.biCompression = BI_RGB;
			//
			myBitmap = CreateCompatibleBitmap(GetDC(hwnd), WIDTH, HEIGHT);
			myCompatibleDC = CreateCompatibleDC(GetDC(hwnd));
			SelectObject(myCompatibleDC, myBitmap);
			break;

    	case WM_ERASEBKGND:
    		return 1;

		case WM_DESTROY:
			DeleteTree();
    		DeleteObject(myBitmap) ;
    	    DeleteDC(myCompatibleDC);
    	    DeleteDC(hdc);
			PostQuitMessage(0);
			exit(0);
			break;

		case WM_LBUTTONDOWN:
		{
			fDraw = TRUE;
			xi = LOWORD(lparam) - WIDTH / 2;
			yi = HIWORD(lparam) - HEIGHT / 2;
			//
			p0.x = position.x;
			p0.y = position.y;
			p0.z = position.z;
			break;
		}

		case WM_LBUTTONUP:
			fDraw = FALSE;
			break;

		case  WM_MOUSEMOVE:
			if(fDraw)
			{
				xf = LOWORD(lparam) - WIDTH / 2;
				yf = HIWORD(lparam) - HEIGHT / 2;
				//
				double dx = xf - xi;
				double dy = yf - yi;
				double h = dx / WIDTH;
				double v = dy / WIDTH;
				//
				position.x = p0.x;
				position.y = p0.y;
				position.z = p0.z;
				rotate(&position, attitude, h);
				//
				Vector3d normal;
				crossproduct(direction, attitude, &normal);
				rotate(&position, normal, v);
				//
				normalize(&position);
				scale3d(&position, 1.5 * GRID);
				//
				double hy = sqrt(position.x * position.x + position.y * position.y + position.z * position.z);
				direction.x = -position.x / hy;
				direction.y = -position.y / hy;
				direction.z = -position.z / hy;
			}
			break;

		case WM_KEYDOWN:
			if(wparam == 0x41)
				showAxes = !showAxes;
			if(wparam == 0x42)
				background ^= 0x00ffffff;
			if(wparam == 0x47)
				showGrid = !showGrid;
			if(wparam > 0x30 && wparam < 0x39)
			{
				types[wparam - 0x30] = !types[wparam - 0x30];
				generateTree();
			}
			if(wparam == 33)
			{
				if(n < 16)
					n++;
				generateTree();
			}
			if(wparam == 34)
			{
				if(n > 1)
					n--;
				generateTree();
			}
			if(wparam == 81)
			{
				quadrant = !quadrant;
				generateTree();
			}
			break;

		default:
			return DefWindowProc(hwnd, msg, wparam, lparam);
	}
	return lparam;
}
